import 'dart:developer';

import 'package:idkit/idkit.dart';

class IDKitLog with ILog, CLog {
  ///
  late LogOption optionLog;

  /// Log Management Simple Interest.
  static IDKitLog? _instance;
  static IDKitLog get instance => _instance ??= IDKitLog._init();

  /// Factory mode creates log management objects.
  factory IDKitLog.init({LogOption? option}) =>
      _instance ??= IDKitLog._init(option: option);

  /// Log management object core initialization method.
  IDKitLog._init({LogOption? option}) {
    optionLog = const LogOption().copy(option);
  }

  static value(Object? value, {String? mark, LogOption? option}) {
    final LogOption vOption = IDKitLog.instance.optionLog.copy(option);
    if (vOption.isEnable ?? true && value.isNoNull) {
      final StackTrace stackTrace = StackTrace.current;
      log(ILog.getHeaderAndFooter(vOption));
      log(ILog.getMark(vOption, mark ?? 'idkit_mark'));
      log(ILog.getWidget(vOption, ILog.getWidgetName(stackTrace)));
      log(ILog.getMethod(vOption, ILog.getCallMethod(stackTrace)));
      log(ILog.getPath(vOption, ILog.getFilePath(stackTrace)));
      log(ILog.getCount(vOption, ILog.getLine(stackTrace)));
      log(ILog.getSupport(vOption));
      log(ILog.getSplitLine(vOption));
      log(ILog.getContent(vOption, value ?? ''));
      log(ILog.getHeaderAndFooter(vOption, type: false));
    }
  }

  static list(Object? value) {
    if (IDKitLog.instance.optionLog.isEnable ?? true) {}
  }

  static map(Object? value) {
    if (IDKitLog.instance.optionLog.isEnable ?? true) {}
  }
}

mixin ILog {
  /// Get the current call log widget name.
  static String getWidgetName(StackTrace stackTrace) {
    final String outPut = stackTrace.toString().split('\n')[1];
    final int widgetStart = outPut.indexOf(RegExp(r'[A-Za-z]'), 1);
    final int widgetEnd = outPut.indexOf('.', widgetStart);
    return outPut.substring(widgetStart, widgetEnd);
  }

  /// Method to get the current call log.
  static String getCallMethod(StackTrace stackTrace) {
    final String outPut = stackTrace.toString().split('\n')[1];
    final int methodStart = outPut.indexOf('.', 1);
    final int methodEnd = outPut.indexOf('.<', methodStart);
    return outPut.substring(methodStart + 1, methodEnd);
  }

  /// Get the file path of the current call log.
  static String getFilePath(StackTrace stackTrace) {
    final String outPut = stackTrace.toString().split('\n')[1];
    final int filePathStart = outPut.indexOf('(', 1);
    final int filePathEnd = outPut.indexOf(RegExp(r':[0-9]'), filePathStart);
    return outPut.substring(filePathStart + 1, filePathEnd);
  }

  /// Get the number of lines in the current call log.
  static String getLine(StackTrace stackTrace) {
    final String outPut = stackTrace.toString().split('\n')[1];
    final int lineStart = outPut.indexOf(RegExp(r'[0-9]+:'), 1);
    final int lineEnd = outPut.indexOf(RegExp(r':[0-9]'), lineStart);
    return outPut.substring(lineStart, lineEnd);
  }

  /// Head  and Footer of log
  static String getHeaderAndFooter(LogOption option, {bool type = true}) {
    final String content = type
        ? (option.language == Language.en ? 'LOG OUTPUT START' : '日志输出开始')
        : (option.language == Language.en ? ' LOG OUTPUT END ' : '日志输出结束');
    final String header = '${'=' * 20} $content ${'=' * 20}';
    int color = 0;
    switch (option.colorTheme) {
      case ColorTheme.red:
        color = 31;
        break;
      default:
    }
    return '\x1B[$color\\m$header\x1B[0m';
  }

  /// Head  and Footer of log
  static String getSupport(LogOption option) {
    final String content = option.language == Language.en
        ? 'SUPPORT: Served by IDKit '
        : '支持: 由 IDKit 提供服务';
    int color = 0;
    switch (option.colorTheme) {
      case ColorTheme.red:
        color = 31;
        break;
      default:
    }
    return '\x1B[$color\\m$content\x1B[0m';
  }

  /// log output split line
  static String getSplitLine(LogOption option) {
    final String content = option.language == Language.en ? '-' * 59 : '-' * 48;
    int color = 0;
    switch (option.colorTheme) {
      case ColorTheme.red:
        color = 31;
        break;
      case ColorTheme.green:
        color = 31;
        break;
      default:
    }
    return '\x1B[$color\\m$content\x1B[0m';
  }

  /// Log output flag.
  static String getMark(LogOption option, Object object) {
    final String content =
        option.language == Language.en ? '   MARK: $object' : '标识: $object';
    int color = 0;
    switch (option.colorTheme) {
      case ColorTheme.red:
        color = 31;
        break;
      default:
    }
    return '\x1B[$color\\m$content\x1B[0m';
  }

  /// Log output widget.
  static String getWidget(LogOption option, Object object) {
    final String content =
        option.language == Language.en ? ' WIDGET: $object' : '组件: $object';
    int color = 0;
    switch (option.colorTheme) {
      case ColorTheme.red:
        color = 31;
        break;
      default:
    }
    return '\x1B[$color\\m$content\x1B[0m';
  }

  /// Log output widget.
  static String getMethod(LogOption option, Object object) {
    final String content =
        option.language == Language.en ? ' METHOD: $object' : '方法: $object';
    int color = 0;
    switch (option.colorTheme) {
      case ColorTheme.red:
        color = 31;
        break;
      default:
    }
    return '\x1B[$color\\m$content\x1B[0m';
  }

  /// Log output widget.
  static String getPath(LogOption option, Object object) {
    final String content =
        option.language == Language.en ? '   PATH: $object' : '路径: $object';
    int color = 0;
    switch (option.colorTheme) {
      case ColorTheme.red:
        color = 31;
        break;
      default:
    }
    return '\x1B[$color\\m$content\x1B[0m';
  }

  /// Log output widget.
  static String getCount(LogOption option, Object object) {
    final String content =
        option.language == Language.en ? '   LINE: $object' : '行数: $object';
    int color = 0;
    switch (option.colorTheme) {
      case ColorTheme.red:
        color = 31;
        break;
      default:
    }
    return '\x1B[$color\\m$content\x1B[0m';
  }

  /// Log output widget.
  static String getContent(LogOption option, Object object) {
    final String content =
        option.language == Language.en ? 'CONTENT: $object' : '内容: $object';
    int color = 0;
    switch (option.colorTheme) {
      case ColorTheme.red:
        color = 31;
        break;
      default:
    }
    return '\x1B[$color\\m$content\x1B[0m';
  }
}

mixin CLog {
  static red(Object? value) => log('\x1B[31m$value\x1B[0m');
  static yellow(Object? value) => log('\x1B[33m$value\x1B[0m');
  static green(Object? value) => log('\x1B[32m$value\x1B[0m');
  static blue(Object? value) => log('\x1B[34m$value\x1B[0m');
  static cyan(Object? value) => log('\x1B[36m$value\x1B[0m');
  static white(Object? value) => log('\x1B[37m$value\x1B[0m');
  static black(Object? value) => log('\x1B[30m$value\x1B[0m');
}
